@@ -2,7 +2,7 @@
  * The XML and API file generator module for SIP.
  *
  * Copyright (c) 2007
- * 	Riverbank Computing Limited <info@riverbankcomputing.co.uk>
+ * 	Phil Thompson <phil@river-bank.demon.co.uk>
  * 
  * This file is part of SIP.
  * 
@@ -396,6 +396,9 @@ static void xmlClass(sipSpec *pt, moduleDef *mod, classDef *cd, FILE *fp)
     prScopedPythonName(fp, cd->ecd, cd->pyname);
     fprintf(fp, "\"");
 
+    if (cd->picklecode != NULL)
+        fprintf(fp, " pickle=\"1\"");
+
     if (cd->convtocode != NULL)
         fprintf(fp, " convert=\"1\"");
 
@@ -688,7 +691,7 @@ static int xmlOverload(classDef *scope, memberDef *md, overDef *od,
 static void xmlCppSignature(FILE *fp, overDef *od)
 {
     prcode(fp, "%M");
-    prOverloadDecl(fp, od, TRUE);
+    prOverloadDecl(fp, NULL, od, TRUE);
     prcode(fp, "%M");
 }
 
