@@ -2,7 +2,7 @@
  * The main header file for SIP.
  *
  * Copyright (c) 2007
- * 	Riverbank Computing Limited <info@riverbankcomputing.co.uk>
+ * 	Phil Thompson <phil@river-bank.demon.co.uk>
  * 
  * This file is part of SIP.
  * 
@@ -71,7 +71,6 @@
 
 /* Handle class flags.  These are combined with the section flags. */
 
-#define CLASS_HAS_ENUMS     0x00000100  /* It has enums. */
 #define CLASS_HAS_SIGSLOTS  0x00000200  /* It has signals or slots. */
 #define CLASS_IS_ABSTRACT   0x00000400  /* It is an abstract class. */
 #define CLASS_HAS_SHADOW    0x00000800  /* It is has a shadow class. */
@@ -90,8 +89,6 @@
 #define CLASS_DTOR_HOLD_GIL 0x01000000  /* The dtor holds the GIL. */
 #define CLASS_QT_META_TYPE  0x02000000  /* Register as a Qt meta type. */
 
-#define hasEnums(cd)        ((cd)->classflags & CLASS_HAS_ENUMS)
-#define setHasEnums(cd)     ((cd)->classflags |= CLASS_HAS_ENUMS)
 #define hasSigSlots(cd)     ((cd)->classflags & CLASS_HAS_SIGSLOTS)
 #define setHasSigSlots(cd)  ((cd)->classflags |= CLASS_HAS_SIGSLOTS)
 #define isAbstractClass(cd) ((cd)->classflags & CLASS_IS_ABSTRACT)
@@ -212,6 +209,9 @@
 #define OVER_DONT_DEREF_SELF    0x00040000  /* For comparison operators, don't dereference self. */
 #define OVER_HOLD_GIL       0x00080000  /* The function holds the GIL. */
 #define OVER_RELEASE_GIL    0x00100000  /* The function releases the GIL. */
+#define OVER_THIS_XFERRED   0x00200000  /* Ownership of this is transferred. */
+#define OVER_IS_GLOBAL      0x00400000  /* It is a global operator. */
+#define OVER_IS_COMPLEMENTARY   0x00800000  /* It is a complementary operator. */
 
 #define isPublic(o)         ((o)->overflags & SECT_IS_PUBLIC)
 #define setIsPublic(o)      ((o)->overflags |= SECT_IS_PUBLIC)
@@ -226,6 +226,7 @@
 
 #define isVirtual(o)        ((o)->overflags & OVER_IS_VIRTUAL)
 #define setIsVirtual(o)     ((o)->overflags |= OVER_IS_VIRTUAL)
+#define resetIsVirtual(o)   ((o)->overflags &= ~OVER_IS_VIRTUAL)
 #define isAbstract(o)       ((o)->overflags & OVER_IS_ABSTRACT)
 #define setIsAbstract(o)    ((o)->overflags |= OVER_IS_ABSTRACT)
 #define isConst(o)          ((o)->overflags & OVER_IS_CONST)
@@ -251,6 +252,12 @@
 #define setIsHoldGIL(o)     ((o)->overflags |= OVER_HOLD_GIL)
 #define isReleaseGIL(o)     ((o)->overflags & OVER_RELEASE_GIL)
 #define setIsReleaseGIL(o)  ((o)->overflags |= OVER_RELEASE_GIL)
+#define isThisTransferredMeth(o)    ((o)->overflags & OVER_THIS_XFERRED)
+#define setIsThisTransferredMeth(o) ((o)->overflags |= OVER_THIS_XFERRED)
+#define isGlobal(o)         ((o)->overflags & OVER_IS_GLOBAL)
+#define setIsGlobal(o)      ((o)->overflags |= OVER_IS_GLOBAL)
+#define isComplementary(o)  ((o)->overflags & OVER_IS_COMPLEMENTARY)
+#define setIsComplementary(o)   ((o)->overflags |= OVER_IS_COMPLEMENTARY)
 
 
 /* Handle variable flags. */
@@ -278,6 +285,7 @@
 #define ARG_IN              0x0200  /* It passes an argument. */
 #define ARG_OUT             0x0400  /* It returns a result. */
 #define ARG_CONSTRAINED     0x0800  /* Suppress type conversion. */
+#define ARG_SINGLE_SHOT     0x1000  /* The slot is only ever fired once. */
 
 #define isReference(a)      ((a)->argflags & ARG_IS_REF)
 #define setIsReference(a)   ((a)-> argflags |= ARG_IS_REF)
@@ -306,6 +314,7 @@
 #define isConstrained(a)    ((a)->argflags & ARG_CONSTRAINED)
 #define setIsConstrained(a) ((a)->argflags |= ARG_CONSTRAINED)
 #define resetIsConstrained(a)   ((a)->argflags &= ~ARG_CONSTRAINED)
+#define isSingleShot(a)     ((a)->argflags & ARG_SINGLE_SHOT)
 
 
 /* Handle name flags. */
@@ -437,7 +446,8 @@ typedef enum {
     anyslot_type,
     cbool_type,
     sstring_type,
-    wstring_type
+    wstring_type,
+    fake_void_type
 } argType;
 
 
@@ -507,7 +517,7 @@ typedef struct _scopedNameDef {
 typedef struct _nameDef {
     int nameflags;                      /* The name flags. */
     struct _moduleDef *module;          /* The main module. */
-    char *text;                         /* The text of the name. */
+    const char *text;                   /* The text of the name. */
     struct _nameDef *next;              /* Next in the list. */
 } nameDef;
 
@@ -807,7 +817,6 @@ typedef struct _enumDef {
     nameDef *pyname;                    /* The Python name (may be NULL). */
     int enumnr;                         /* The enum number. */
     struct _classDef *ecd;              /* The enclosing class. */
-    struct _classDef *pcd;              /* The publishing class. */
     moduleDef *module;                  /* The owning module. */
     enumMemberDef *members;             /* The list of members. */
     struct _memberDef *slots;           /* The list of slots. */
@@ -995,7 +1004,8 @@ void appendCodeBlock(codeBlock **headp, codeBlock *new);
 void prcode(FILE *fp, const char *fmt, ...);
 void prOverloadName(FILE *fp, overDef *od);
 void prScopedPythonName(FILE *fp, classDef *scope, const char *pyname);
-void prOverloadDecl(FILE *fp, overDef *od, int defval);
+void prOverloadDecl(FILE *fp, classDef *context, overDef *od, int defval);
+void searchTypedefs(sipSpec *pt, scopedNameDef *snd, argDef *ad);
 int isIntReturnSlot(memberDef *md);
 int isLongReturnSlot(memberDef *md);
 int isVoidReturnSlot(memberDef *md);
@@ -1010,6 +1020,7 @@ int optNoEmitters(sipSpec *pt);
 int optRegisterTypes(sipSpec *pt);
 int optQ_OBJECT4(sipSpec *pt);
 void yywarning(char *);
+nameDef *cacheName(sipSpec *pt, const char *name);
 
 
 /* These are only here because bison publically references them. */
