@@ -2,7 +2,7 @@
  * The parse tree transformation module for SIP.
  *
  * Copyright (c) 2007
- * 	Riverbank Computing Limited <info@riverbankcomputing.co.uk>
+ * 	Phil Thompson <phil@river-bank.demon.co.uk>
  * 
  * This file is part of SIP.
  * 
@@ -64,7 +64,7 @@ static void fatalNoDefinedType(scopedNameDef *);
 static void getBaseType(sipSpec *,moduleDef *,classDef *,argDef *);
 static void searchScope(sipSpec *,classDef *,scopedNameDef *,argDef *);
 static void searchMappedTypes(sipSpec *,scopedNameDef *,argDef *);
-static void searchTypedefs(sipSpec *,scopedNameDef *,argDef *);
+void searchTypedefs(sipSpec *pt, scopedNameDef *snd, argDef *ad);
 static void searchEnums(sipSpec *,scopedNameDef *,argDef *);
 static void searchClasses(sipSpec *,moduleDef *mod,scopedNameDef *,argDef *);
 static void appendToMRO(mroDef *,mroDef ***,classDef *);
@@ -78,6 +78,9 @@ static mappedTypeDef *instantiateMappedTypeTemplate(sipSpec *pt, moduleDef *mod,
 static classDef *getProxy(moduleDef *mod, classDef *cd);
 static int generatingCodeForModule(sipSpec *pt, moduleDef *mod);
 static void registerMetaType(classDef *cd);
+static void addComplementarySlots(sipSpec *pt, classDef *cd);
+static void addComplementarySlot(sipSpec *pt, classDef *cd, memberDef *md,
+        slotType cslot, const char *cslot_name);
 
 
 /*
@@ -228,11 +231,23 @@ void transform(sipSpec *pt)
      * Move casts and slots around to their correct classes (if in the same
      * module) or create proxies for them (if cross-module).
      */
-    if (!pt -> genc)
+    if (!pt->genc)
         for (mod = pt->modules; mod != NULL; mod = mod->next)
             if (generatingCodeForModule(pt, mod))
                 moveMainModuleCastsSlots(pt, mod);
 
+    /* Automatically generate missing complementary slots. */
+    if (!pt->genc)
+    {
+        for (cd = pt->classes; cd != NULL; cd = cd->next)
+            addComplementarySlots(pt, cd);
+
+        for (mod = pt->modules; mod != NULL; mod = mod->next)
+            if (generatingCodeForModule(pt, mod))
+                for (cd = mod->proxies; cd != NULL; cd = cd->next)
+                    addComplementarySlots(pt, cd);
+    }
+
     /* Generate the different class views. */
     for (cd = pt->classes; cd != NULL; cd = cd->next)
         if (cd->iff->type == class_iface)
@@ -275,6 +290,107 @@ void transform(sipSpec *pt)
 }
 
 
+/*
+ * Add any missing complementary slots to a class.  This emulates the C++
+ * behaviour of automatically interpreting (for example) >= as !<.
+ */
+static void addComplementarySlots(sipSpec *pt, classDef *cd)
+{
+    memberDef *md;
+
+    for (md = cd->members; md != NULL; md = md->next)
+        switch (md->slot)
+        {
+        case lt_slot:
+            addComplementarySlot(pt, cd, md, ge_slot, "__ge__");
+            break;
+
+        case le_slot:
+            addComplementarySlot(pt, cd, md, gt_slot, "__gt__");
+            break;
+
+        case gt_slot:
+            addComplementarySlot(pt, cd, md, le_slot, "__le__");
+            break;
+
+        case ge_slot:
+            addComplementarySlot(pt, cd, md, lt_slot, "__lt__");
+            break;
+
+        case eq_slot:
+            addComplementarySlot(pt, cd, md, ne_slot, "__ne__");
+            break;
+
+        case ne_slot:
+            addComplementarySlot(pt, cd, md, eq_slot, "__eq__");
+            break;
+        }
+}
+
+
+/*
+ * Add a complementary slot if it is missing.
+ */
+static void addComplementarySlot(sipSpec *pt, classDef *cd, memberDef *md,
+        slotType cslot, const char *cslot_name)
+{
+    overDef *od1;
+    memberDef *md2 = NULL;
+
+    for (od1 = cd->overs; od1 != NULL; od1 = od1->next)
+    {
+        overDef *od2;
+
+        if (od1->common != md || isComplementary(od1) || od1->methodcode != NULL)
+            continue;
+
+        /* Try and find an existing complementary slot. */
+        for (od2 = cd->overs; od2 != NULL; od2 = od2->next)
+            if (od2->common->slot == cslot && sameSignature(&od1->pysig, &od2->pysig, TRUE))
+                break;
+
+        /*
+         * If there is an explicit complementary slot then there is nothing to
+         * do.
+         */
+        if (od2 != NULL)
+            continue;
+
+        /* Create a new member if needed. */
+        if (md2 == NULL)
+        {
+            for (md2 = cd->members; md2 != NULL; md2 = md2->next)
+                if (md2->slot == cslot)
+                    break;
+
+            if (md2 == NULL)
+            {
+                md2 = sipMalloc(sizeof (memberDef));
+
+                md2->pyname = cacheName(pt, cslot_name);
+                md2->memberflags = md->memberflags;
+                md2->slot = cslot;
+                md2->module = md->module;
+
+                md2->next = cd->members;
+                cd->members = md2;
+            }
+        }
+
+        /* Create the complementary slot. */
+        od2 = sipMalloc(sizeof (overDef));
+
+        *od2 = *od1;
+        resetIsVirtual(od2);
+        setIsComplementary(od2);
+        od2->common = md2;
+
+        od2->next = cd->overs;
+        cd->overs = od2;
+    }
+}
+
+
 /*
  * See if a class needs to be registered as a Qt meta type.
  */
@@ -612,10 +728,7 @@ static void moveGlobalSlot(sipSpec *pt, moduleDef *mod, memberDef *gmd)
         /* Remove from the list. */
         *odp = od->next;
 
-        /*
-         * The only time we need the name of an enum is when it has
-         * slots.
-         */
+        /* The only time we need the name of an enum is when it has slots. */
         if (nd != NULL)
             setIsUsedName(nd);
 
@@ -638,6 +751,7 @@ static void moveGlobalSlot(sipSpec *pt, moduleDef *mod, memberDef *gmd)
 
         /* Move the overload. */
         setIsPublic(od);
+        setIsGlobal(od);
         od->common = md;
         od->next = *odhead;
 
@@ -1260,29 +1374,6 @@ static void getVisibleMembers(sipSpec *pt, classDef *cd)
 
         mrocd = mro->cd;
 
-        /*
-         * If the base class is in the main module, see if it needs to publish
-         * any protected enums.
-         */
-        if (generatingCodeForModule(pt, cd->iff->module))
-        {
-            enumDef *ed;
-
-            for (ed = pt->enums; ed != NULL; ed = ed->next)
-            {
-                /* Skip unless we are the publisher. */
-                if (ed->pcd != mrocd)
-                    continue;
-
-                /*
-                 * If we are not in the main module then the base class must
-                 * take over as the publisher.
-                 */
-                if (!generatingCodeForModule(pt, mrocd->iff->module))
-                    ed->pcd = cd;
-            }
-        }
-
         for (md = mrocd->members; md != NULL; md = md->next)
         {
             visibleList *vl;
@@ -1723,10 +1814,10 @@ static void resolveVariableType(sipSpec *pt, varDef *vd)
         break;
     }
 
-    if (bad)
+    if (bad && (vd->getcode == NULL || vd->setcode == NULL))
     {
         fatalScopedName(vd->fqcname);
-        fatal(" has an unsupported type\n");
+        fatal(" has an unsupported type - provide %%GetCode and %%SetCode\n");
     }
  
     if (vtype->atype != class_type && vd->accessfunc != NULL)
@@ -1741,8 +1832,8 @@ static void resolveVariableType(sipSpec *pt, varDef *vd)
         ifaceFileIsUsed(&vd->module->used, vtype);
 
     /*
-     * Instance variables or static class variables (unless they are
-     * constants) need a handler.
+     * Instance variables or static class variables (unless they are constants)
+     * need a handler.
      */
     if (vd->ecd != NULL && vd->accessfunc == NULL &&
         (!isStaticVar(vd) || vtype->nrderefs != 0 || !isConstArg(vtype)))
@@ -2595,8 +2686,7 @@ static void searchMappedTypes(sipSpec *pt,scopedNameDef *snd,argDef *ad)
 /*
  * Search the typedefs for a name and return the type.
  */
-
-static void searchTypedefs(sipSpec *pt,scopedNameDef *snd,argDef *ad)
+void searchTypedefs(sipSpec *pt, scopedNameDef *snd, argDef *ad)
 {
     typedefDef *td;
 
