@@ -2,7 +2,7 @@
  * The SIP module interface.
  *
  * Copyright (c) 2007
- * 	Riverbank Computing Limited <info@riverbankcomputing.co.uk>
+ * 	Phil Thompson <phil@river-bank.demon.co.uk>
  * 
  * This file is part of SIP.
  * 
@@ -51,8 +51,8 @@ extern "C" {
 /*
  * Define the SIP version number.
  */
-#define SIP_VERSION         0x040700
-#define SIP_VERSION_STR     "4.7"
+#define SIP_VERSION         0x040703
+#define SIP_VERSION_STR     "4.7.3"
 
 
 /*
@@ -65,6 +65,8 @@ extern "C" {
  *
  * History:
  *
+ * 3.6  Added the 'g' format character to sip_api_parse_args().
+ *
  * 3.5  Added the td_pickle field to the sipTypeDef structure.
  *      Added sip_api_transfer_break().
  *
@@ -112,7 +114,7 @@ extern "C" {
  * 0.0  Original version.
  */
 #define SIP_API_MAJOR_NR    3
-#define SIP_API_MINOR_NR    5
+#define SIP_API_MINOR_NR    6
 
 
 /* Some compatibility stuff to help with handwritten code for SIP v3. */
@@ -1255,6 +1257,8 @@ typedef struct _sipQtAPI {
     void (*qt_forget_sender)();
     int (*qt_same_name)(const char *, const char *);
     sipSlotConnection *(*qt_find_connection)(void *, void **);
+    void *(*qt_create_universal_slot_ex)(struct _sipWrapper *,
+            struct _sipSlotConnection *, const char **, int);
 } sipQtAPI;
 
 
@@ -1274,6 +1278,12 @@ typedef struct _sipQtAPI {
 #define SIP_DERIVED_CLASS   0x0002  /* The instance is derived. */
 
 
+/*
+ * These flags are specific to the Qt support API.
+ */
+#define SIP_SINGLE_SHOT     0x01    /* The connection is single shot. */
+
+
 /*
  * Useful macros, not part of the public API.
  */
@@ -1283,6 +1293,7 @@ typedef struct _sipQtAPI {
 #define SIP_NOT_IN_MAP      0x0020  /* If Python object not in the map. */
 #define SIP_SHARE_MAP       0x0040  /* If the map slot might be occupied. */
 #define SIP_CPP_HAS_REF     0x0080  /* If C/C++ has a reference. */
+#define SIP_POSSIBLE_PROXY  0x0100  /* If there might be a proxy slot. */
 
 #define sipIsPyOwned(w)     ((w)->flags & SIP_PY_OWNED)
 #define sipSetPyOwned(w)    ((w)->flags |= SIP_PY_OWNED)
@@ -1294,7 +1305,9 @@ typedef struct _sipQtAPI {
 #define sipSetNotInMap(w)   ((w)->flags |= SIP_NOT_IN_MAP)
 #define sipCppHasRef(w)     ((w)->flags & SIP_CPP_HAS_REF)
 #define sipSetCppHasRef(w)  ((w)->flags |= SIP_CPP_HAS_REF)
-#define sipResetCppHasRef(w) ((w)->flags &= ~SIP_CPP_HAS_REF)
+#define sipResetCppHasRef(w)    ((w)->flags &= ~SIP_CPP_HAS_REF)
+#define sipPossibleProxy(w) ((w)->flags & SIP_POSSIBLE_PROXY)
+#define sipSetPossibleProxy(w)  ((w)->flags |= SIP_POSSIBLE_PROXY)
 
 
 #define SIP_TYPE_ABSTRACT   0x0001  /* If the type is abstract. */
